package com.ofss.ams.services;


import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.mail.MessagingException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Service;
import org.thymeleaf.context.Context;
import org.thymeleaf.spring5.SpringTemplateEngine;

import com.ofss.ams.domain.EmailObj;

@Service
public class MailServiceImpl implements MailService {

	@Autowired
	JavaMailSender mailSender;
	
	@Autowired
    private SpringTemplateEngine thymeleafTemplateEngine;
	
	Logger logger = LoggerFactory.getLogger(MailServiceImpl.class);

	public void sendEmail(EmailObj mail) {

		logger.info("Mail Object " + mail.toString());
		logger.info("Inside MailServiceImpl.sendEmail with Mail Object " + mail.getMailTo());

		MimeMessage mimeMessage = mailSender.createMimeMessage();

		try {

			MimeMessageHelper mimeMessageHelper = new MimeMessageHelper(mimeMessage, true);

			mimeMessageHelper.setSubject(mail.getMailSubject("Test Mail - Ignore"));
			mimeMessageHelper.setFrom(new InternetAddress(mail.getMailFrom(), "no-reply-OBAMS"));
			mimeMessageHelper.setTo(mail.getMailTo());
			mimeMessageHelper.setText(mail.getMailContent());

			// Optional
			mimeMessageHelper.setCc(mail.getMailCc());

			mailSender.send(mimeMessageHelper.getMimeMessage());

		} catch (MessagingException e) {
			logger.info("Inside Catch of : MessagingException, printing trace..");
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			logger.info("Inside Catch of : UnsupportedEncodingException, printing trace..");
			e.printStackTrace();
		}

	}

	public void sendMessageUsingThymeleafTemplate(EmailObj mail) throws IOException, MessagingException {

		logger.info("Inside sendMessageUsingThymeleafTemplate");
		logger.info("Inside MailServiceImpl.sendMessageUsingThymeleafTemplate with Mail Object " + mail.getMailTo());

		Map<String, Object> templateModel = new HashMap<String, Object>();
		
		templateModel.put("mailContent", mail.getMailContent());
		templateModel.put("source", mail.getSource());
		templateModel.put("env", mail.getEnvironment());
		templateModel.put("timestamp", new Date());
		templateModel.put("severe", mail.getSeverity());
		
		Context thymeleafContext = new Context();
		thymeleafContext.setVariables(templateModel);
		
		String htmlBody;
		logger.info("template type: {}", mail.getTemplate());
		
		if(mail.getTemplate().equals("CUSTOMER")) {
			logger.info("Inside CUSTOMER Template");
			htmlBody = thymeleafTemplateEngine.process("template-customer.html", thymeleafContext);
		}
		else {
			logger.info("Inside INTERNAL Template");
			htmlBody = thymeleafTemplateEngine.process("template-internal.html", thymeleafContext);
		}
		
//		sendHtmlMessage(mail, mail.getMailTo(), mail.getMailCc(), mail.getMailSubject("Test Mail - Ignore"), htmlBody);
	}
	
}
